package coop.libriciel.groovyrunner;

import com.fasterxml.jackson.databind.ObjectMapper;
import coop.libriciel.groovyrunner.model.CircuitRequest;
import coop.libriciel.groovyrunner.model.CircuitResponse;
import coop.libriciel.service.DeskUtilsService;
import lombok.Getter;
import lombok.Setter;
import org.alfresco.repo.cache.TransactionalCache;
import org.alfresco.model.ContentModel;
import org.alfresco.repo.security.authentication.AuthenticationUtil;
import org.alfresco.service.ServiceRegistry;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationEvent;
import org.springframework.extensions.surf.util.AbstractLifecycleBean;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.List;
import java.util.concurrent.*;
import java.util.stream.Collectors;

@Service
@Getter
@Setter
public class GroovyRunnerService extends AbstractLifecycleBean {

    @Value("${parapheur.groovyrunner.connector.host}")
    private String connectorHost;
    @Value("${parapheur.groovyrunner.connector.port}")
    private String connectorPort;
    @Value("${parapheur.groovyrunner.connector.path}")
    private String connectorPath;

    @Autowired
    DeskUtilsService deskUtilsService;

    @Autowired
    private ServiceRegistry serviceRegistry;

    private Future<List<String>> future;
    private ObjectMapper mapper;

    protected void init() {
        mapper = new ObjectMapper();
    }

    public CircuitResponse selectWorkflow(CircuitRequest request) {
        CircuitResponse response = new CircuitResponse();
        response.setError("No workflow");
        try {
            request.setDesks(future.get());
            response = doRequest(request);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return response;
    }

    private CircuitResponse doRequest(CircuitRequest request) throws Exception {
        CircuitResponse circuitResponse = new CircuitResponse();
        circuitResponse.setError("No workflow");

        CloseableHttpClient httpclient = HttpClients.createDefault();

        HttpPost post = new HttpPost("http://" + connectorHost + ":" + connectorPort + connectorPath);
        post.setEntity(new StringEntity(mapper.writeValueAsString(request), "UTF-8"));
        post.setHeader("Content-type", "application/json");

        // Execute and get the response.
        try {
            HttpResponse response = httpclient.execute(post);
            HttpEntity entity = response.getEntity();
            System.out.println(mapper.writeValueAsString(request));
            System.out.println("Réponse du serveur : " + response.getStatusLine().getStatusCode());
            if (entity != null && response.getStatusLine().getStatusCode() == 200) {
                circuitResponse = mapper.readValue(entity.getContent(), CircuitResponse.class);
            } else {
                throw new Exception("Cannot stamp document, see error log on pdf-stamp application");
            }
        } catch (IOException e) {
            e.printStackTrace();
            throw e;
        }
        httpclient.close();
        return circuitResponse;
    }

    // TODO - Handle multi-coll
    public void refreshDesksCache() {
        if(future == null || future.isDone()) {
            // Exécution du script
            Callable<List<String>> task = () -> {
                return AuthenticationUtil.runAs(() -> {
                    return deskUtilsService.listDesks().stream().map(nodeRef ->
                                    serviceRegistry.getNodeService().getProperty(nodeRef, ContentModel.PROP_TITLE).toString())
                            .collect(Collectors.toList());
                }, AuthenticationUtil.getAdminUserName());
            };

            ExecutorService executor = Executors.newSingleThreadExecutor();
            future = executor.submit(task);
        }
    }

    @Override
    protected void onBootstrap(ApplicationEvent applicationEvent) {
        refreshDesksCache();
    }

    @Override
    protected void onShutdown(ApplicationEvent applicationEvent) {

    }
}
